/*
 ============================================================================

  This program is free software, released under the GNU General Public License
  (GPL version 2 or later).

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

  http://www.fsf.org/licenses/gpl.html

 ============================================================================
*/

#include "BBTaskMenu.h"
#include "sysmenu.cpp"

char *plugin_info[] =
{
	"BBTaskMenu 0.0.4",
	"BBTaskMenu",
	"0.0.4",
	"ysuke",
	"2006-02-28",
	"http://zzbb.hp.infoseek.co.jp/",
	"zb2_460@yahoo.co.jp",

	"@BBTaskMenu.ShowMenu"
	"@BBTaskMenu.ShowSysMenu"
};

#define szVersion       plugin_info[0]
#define szAppName       plugin_info[1]
#define szInfoVersion	plugin_info[2]
#define szInfoAuthor	plugin_info[3]
#define szInfoRelDate	plugin_info[4]
#define szInfoLink	plugin_info[5]
#define szInfoEmail	plugin_info[6]
#define szBroamList	plugin_info[7]


//===========================================================================

int beginPlugin(HINSTANCE hPluginInstance)
{
	WNDCLASS wc;
	hwndBlackbox = GetBBWnd();
	hInstance = hPluginInstance;

	// Register the window class...
	ZeroMemory(&wc,sizeof(wc));
	wc.lpfnWndProc = WndProc;			// our window procedure
	wc.hInstance = hPluginInstance;		// hInstance of .dll
	wc.lpszClassName = szAppName;		// our window class name
	if (!RegisterClass(&wc)) 
	{
		MessageBox(hwndBlackbox, "Error registering window class", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}

	// Create the window...
	hwndPlugin = CreateWindowEx(
						WS_EX_TOOLWINDOW,								// window style
						szAppName,										// our window class name
						NULL,											// NULL -> does not show up in task manager!
						0,
						0, 0, 0, 0,											// window height
						NULL,											// parent window
						NULL,											// no menu
						hPluginInstance,								// hInstance of .dll
						NULL);
	if (!hwndPlugin)
	{						   
		MessageBox(0, "Error creating window", szAppName, MB_OK | MB_ICONERROR | MB_TOPMOST);
		// Unregister Blackbox messages...
		SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);
		return 1;
	}
	// Register to receive Blackbox messages...
	SendMessage(hwndBlackbox, BB_REGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);

	return 0;
}

//===========================================================================

void endPlugin(HINSTANCE hPluginInstance)
{
        DestroyWindow(hwndPlugin);
        SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwndPlugin, (LPARAM)msgs);
        UnregisterClass(szAppName, hPluginInstance);
}

//===========================================================================

LPCSTR pluginInfo(int field)
{
	switch (field)
	{
		default:
		case 0: return szVersion;
		case 1: return szAppName;
		case 2: return szInfoVersion;
		case 3: return szInfoAuthor;
		case 4: return szInfoRelDate;
		case 5: return szInfoLink;
		case 6: return szInfoEmail;
		case 7: return szBroamList;
	}
}

//===========================================================================

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case BB_TASKSUPDATE:
			ShowMenu(MakeTaskMenu(false));
			ShowMenu(MakeTasksSysMenu(false));
			break;

		case BB_BROADCAST:
		{
			szTemp = (char*)lParam;

			if (strnicmp(szTemp, "@BBTaskMenu.", 12))
				return 0;
			szTemp += 12;

			if (!_strnicmp(szTemp, "Activate", 8)){
                                activated_task = atoi(szTemp + 9);
                                HWND hw = GetTask(activated_task);

				if(activated_task == active_task && !IsIconic(hw)){
					PostMessage(hwndBlackbox, BB_WINDOWMINIMIZE, 0, (LPARAM)hw);
					break;
				}

				PostMessage(hwndBlackbox, BB_BRINGTOFRONT, 0,  (LPARAM)hw);
				ShowMenu(MakeTaskMenu(false));
			}
			if (!_strnicmp(szTemp, "SysMenu", 7)){
				HWND hw = GetTask(atoi(szTemp + 8));

				ShowSysmenu(hw, hwnd);
				ShowMenu(MakeTasksSysMenu(false));
			}
			if (!_stricmp(szTemp, "ShowMenu"))
			{
				ShowMenu(MakeTaskMenu(true));
                        }
			if (!_stricmp(szTemp, "ShowSysMenu"))
			{
				ShowMenu(MakeTasksSysMenu(true));
                        }

                        HWND task_hwnd = NULL;
                        int task_msg = 0;
                        if (2 == sscanf(szTemp, "SysCommand <%d> <%d>", (int*)&task_hwnd, (int*)&task_msg))
                        {
				PostMessage(task_hwnd, WM_SYSCOMMAND, (WPARAM)task_msg, 0);
				ShowMenu(MakeTaskMenu(false));
                        }
		} break;

 		// ==========
		default:
			return DefWindowProc(hwnd,message,wParam,lParam);
	}
	return 0;
}

//===========================================================================

Menu* MakeTaskMenu(bool popup)
{
	active_task = popup ? GetActiveTask() : activated_task;

	Menu *main = MakeNamedMenu("BBTaskMenu", "BBTM_task", popup);

	for (int i = 0; i < GetTaskListSize(); i++){
    		char buf[256];
    		char caption[256];
    		HWND hw = GetTask(i);
    		GetWindowText(hw, caption, 255);

    		sprintf(buf, "@BBTaskMenu.Activate %d", i);
    		MakeMenuItem(main, caption, buf, false);
	}
	return main;
}

//===========================================================================

Menu* MakeTasksSysMenu(bool popup)
{
	Menu *main = MakeNamedMenu("BBTaskMenu(SysMenu)", "BBTM_sysmenu", popup);

	for (int i = 0; i < GetTaskListSize(); i++){
    		char buf[256];
    		char caption[256];
    		HWND hw = GetTask(i);
    		GetWindowText(hw, caption, 255);

    		sprintf(buf, "@BBTaskMenu.SysMenu %d", i);
    		MakeMenuItem(main, caption, buf, false);
	}
	return main;
}

//===========================================================================
